package proxy.util;

import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

public class User {
	private static final Charset UTF_8 = Charset.forName("UTF-8");
	
	private final String username;
	private final byte[] hash;
	private final byte[] salt;
	
	private boolean admin;
	
	public User(String username, byte[] hash, byte[] salt) {
		this.username = username;
		this.hash = hash;
		this.salt = salt;
	}
	
	public String getName() {
		return this.username;
	}
	
	public boolean checkPassword(String password) {
		return MessageDigest.isEqual(hashPassword(password, this.salt), this.hash);
	}
	
	public boolean isAdmin() {
		return this.admin;
	}
	
	public void setAdmin(boolean admin) {
		this.admin = admin;
	}
	
	private static byte[] hashPassword(String pass, byte[] salt) {
		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-512");
			digest.update(pass.getBytes(UTF_8));
			digest.update(salt);
			return digest.digest();
		}
		catch(NoSuchAlgorithmException e) {
			Log.error("Could not generate password hash");
			return null;
		}
	}
	
	public static User createUser(String user, String pass) {
		SecureRandom rand = new SecureRandom();
		byte[] salt = new byte[32];
		rand.nextBytes(salt);
		return new User(user, hashPassword(pass, salt), salt);
	}
}
